//##protect##"FileHeader"
/** @file PCIe1032.h
 *
 *  @authors S. Beuque, C. Guinot, J. Krauth, N. Reibel, J. Rose, L. Sartory, S. Weber
 *  @date    Creation: 05.12.2012
 *  @date    $LastChangedDate: 2013-02-15 13:44:41 +0100 (Fr, 15. Feb 2013) $
 *  @version $LastChangedRevision: 3857 $
 *
 *  @par DESCRIPTION
 *  Main xPCIe1032 header file 
 *
 *  @par LICENCE
 *  @verbatim
 *  Copyright (C) 2009 - 2012 ADDI-DATA GmbH
 *
 *  ADDI-DATA GmbH
 *  Airpark Business Center
 *  Airport Boulevard B210
 *  77836 Rheinmnster
 *  Germany
 *  Tel: +49(0)7229/1847-0    |  Fax: +49(0)7229/1847-200
 *  http://www.addi-data-com  |  info@addi-data.com
 *  @endverbatim
 **/
//##protect##"FileHeader"

#include <windows.h>
#include "define.h"

//##protect##"UserHeaderBegin"
//##protect##"UserHeaderBegin"

#ifdef __cplusplus
extern "C"
{
#endif

/** \defgroup PCIe1032 xPCIe1032 functions
 * @{
 */

/**		\addtogroup General xPCIe1032 General functions
 * 		\ingroup PCIe1032
 *		@{
 */

/* i_PCIe1032_GetNumberOfBoards */
/**
 * Returns the number of xPCIe1032 boards present in the system. \n
 * The function fills the value pointed by pb_NbrOfBoards with the number of boards present.
 *
 * @param[out]  pb_NbrOfBoards Value that will be filled with the number of boards
 * @retval  0   No error
 * @retval -1   Error while getting the number of boards
 */
_INT_ i_PCIe1032_GetNumberOfBoards(OUT BYTE *pb_NbrOfBoards);


/* i_PCIe1032_GetBoardInformation */
/**
 * Gets generic device information.
 *
 * @param[in]   b_BoardIndex Index of the board
 * @param[in]   dw_IdentifierStringSize Size of the identifier parameter buffer
 * @param[out]  pc_Identifier Identifier of the board
 * @param[out]  pdw_UINumber This number is typically a user-perceived slot number, such as a number printed next to the slot on the board, or some other number that makes locating the physical device easier for the user. If the device is on a bus that has no UI number convention, or if the bus driver for the device cannot determine the UI number, this value is 0xFFFFFFFF.
 * @param[out]  pdw_DeviceNumber Device number.
 * @param[out]  pdw_BusNumber Bus number.
 * @param[out]  pdw_BaseAddress0 Base address 0 of the board
 * @param[out]  pdw_BaseAddress1 Base address 1 of the board
 * @param[out]  pdw_BaseAddress2 Base address 2 of the board
 * @param[out]  pdw_BaseAddress3 Base address 3 of the board
 * @param[out]  pb_Interrupt Interrupt line of the board
 * @retval  0   No error
 * @retval -1   Unable to open the board via index
 * @retval -2   Error when calling driver's IOCTL
 */
_INT_ i_PCIe1032_GetBoardInformation(IN  BYTE b_BoardIndex,
                                     IN  DWORD dw_IdentifierStringSize,
                                     OUT CHAR  *pc_Identifier,
                                     OUT DWORD *pdw_UINumber,
                                     OUT DWORD *pdw_DeviceNumber,
                                     OUT DWORD *pdw_BusNumber,
                                     OUT DWORD *pdw_BaseAddress0,
                                     OUT DWORD *pdw_BaseAddress1,
                                     OUT DWORD *pdw_BaseAddress2,
                                     OUT DWORD *pdw_BaseAddress3,
                                     OUT BYTE  *pb_Interrupt);


/* i_PCIe1032_OpenBoardViaIndex */
/**
 * Opens the board with the specified index. A handle is returned to the \n
 * user which allows driver functions to be used. \n
 *
 * @param[in]   b_BoardIndex Index of the board to open
 * @param[out]  ph_DeviceHandle Handle of the selected xPCIe1032 board
 * @retval  0   No error
 * @retval -1   Unable to get the number of available boards
 * @retval -2   Wrong device index parameter, the parameter is too high
 * @retval -3   Error while getting the device's class
 * @retval -4   Error while getting informations about the device
 * @retval -5   Memory allocation error
 * @retval -6   Get device interface detail error
 * @retval -7   Error while creating communication handle
 * @retval -8   Error in the initialization callback function
 */
_INT_ i_PCIe1032_OpenBoardViaIndex(IN BYTE b_BoardIndex, OUT HANDLE *ph_DeviceHandle);


/* i_PCIe1032_OpenBoardViaIdentifier */
/**
 * Opens the board with the specified identifier. A handle is returned to the \n
 * user which allows driver functions to be used. \n
 *
 * @param[in]   pc_Identifier Identifier of the board to open
 * @param[out]  ph_DeviceHandle Handle of the xPCIe1032 board
 * @retval  0   No Error
 * @retval -1   Unable to get the number of available boards
 * @retval -2   Unable to get the board identifier
 * @retval -3   Unable to open the board
 * @retval -4   Identifier not found
 */
_INT_ i_PCIe1032_OpenBoardViaIdentifier(IN CHAR pc_Identifier[], OUT HANDLE *ph_DeviceHandle);


/* i_PCIe1032_CloseBoard */
/**
 * Releases the handle to the board. Block the access to the board.
 *
 * @param[in]   h_DeviceHandle Handle of the board
 * @retval	0   No Error
 * @retval -1   The device handle parameter is wrong
 * @retval -2   Unable to close the device handle
 */
_INT_ i_PCIe1032_CloseBoard(HANDLE h_DeviceHandle);


/* i_PCIe1032_GetDLLVersion */
/**
 * Gets the version of the DLL.
 *
 * @param[out]  pdw_Version Version. For example: 0x01010910 -> Version 1.01, September 2010
 * @retval  0   No Error
 */
_INT_ i_PCIe1032_GetDLLVersion(OUT DWORD *pdw_Version);


/* i_PCIe1032_GetDeviceDriverVersion */
/**
 * Gets the version of the device driver.
 *
 * @param[in]   h_DeviceHandle Handle of the board
 * @param[out]  pdw_Version Version. For example: 0x01011211 -> Version 1.01, December 2011
 * @retval  0   No Error
 * @retval -1   Error while calling IOCTL function
 */
_INT_ i_PCIe1032_GetDeviceDriverVersion(IN HANDLE h_DeviceHandle, OUT DWORD *pdw_Version);


/* pc_PCIe1032_GetLastError */
/**
 * Gets a pointer to a string describing the last error that occurred.
 *
 * @param[in]   h_DeviceHandle Handle of the board
 * @returns a pointer to the error string
 */
const _PCHAR_ pc_PCIe1032_GetLastError(IN HANDLE h_DeviceHandle);


/* i_PCIe1032_SetBoardIntRoutine */
/**
 * Registers an interrupt callback function.
 * This callback function will be called each time the board generates an interruption. \n
 * The value of dw_InterruptSource will enable you to determine the source of the interruption:
 *
 * @param[in]   h_DeviceHandle Handle of the board
 * @param[in]   v_FunctionName Address of the callback function
 *		\n	\b h_DeviceHandle: Handle of the board
 *		\n	\b b_FIFOOverflow: If uc_FIFOOverflow != 0, there are too many unprocessed interrupts in the kernel FIFO, and any new information is lost
 *		\n	\b dw_InterruptMask: Source of the interrupt
 *		\n	\b pw_Args: Values read during the interrupt 
 *		\n	\b dw_ArgsCount: Number of values read during the interrupt
 *
 * The source of the interrupt can be:
 * \li 0x1: Digital input interrupt
 *
 *
 * @retval  0   No Error
 * @retval -1   The device handle parameter is wrong
 * @retval -2   An interrupt routine is already installed. Remove it with ResetBoardIntRoutine and retry
 * @retval -3   Error while creating kill event
 * @retval -4   Error while creating interrupt event
 * @retval -5   Error while creating interrupt thread
 * @retval -6   Error while changing the priority class of the thread
 * @retval -7   Error while changing the priority of the thread
 * @retval -8   Error while calling the IOCTL command
 * @retval -9   Error in the interrupt callback function
 */
_INT_ i_PCIe1032_SetBoardIntRoutine(IN HANDLE h_DeviceHandle,
                                    IN void (*v_FunctionName)(IN  HANDLE h_DeviceHandle,
                                                              IN  BYTE   b_FifoOverflow,
                                                              IN  DWORD  dw_InterruptSource,
                                                              IN  DWORD  dw_InterruptMask,
                                                              OUT DWORD  *pdw_Args,
                                                              IN  DWORD  dw_ArgsCount));


/* i_PCIe1032_ResetBoardIntRoutine */
/**
 * Deregisters the current interrupt callback function.
 *
 * @param[in]   h_DeviceHandle Handle of the board
 * @retval  0   No Error
 * @retval -1   The device handle parameter is wrong
 * @retval -2   No interrupt routine installed
 * @retval -3   Error in the reset interrupt callback function
 */
_INT_ i_PCIe1032_ResetBoardIntRoutine(IN HANDLE h_DeviceHandle);

/**		@}
 */

/**		\addtogroup DigitalInput xPCIe1032 Digital input functions
 * 		\ingroup PCIe1032
 *		@{
 */

/* i_PCIe1032_Read1DigitalInput */
/**
 * Reads 1 digital input of the board
 *
 * @param[in]   h_DeviceHandle Handle of the board
 * @param[in]   b_Channel Channel to be read (0..31)
 * @param[out]  b_Value Status of the digital input
 * @retval  0   No error
 * @retval -1   The h_DeviceHandle parameter is wrong
 * @retval -2   The b_Channel parameter is wrong
 */
_INT_ i_PCIe1032_Read1DigitalInput(IN  HANDLE h_DeviceHandle,
                                   IN  BYTE b_Channel,
                                   OUT BYTE *b_Value);


/* i_PCIe1032_Read4DigitalInputs */
/**
 * Reads 4 digital inputs of the board
 *
 * @param[in]   h_DeviceHandle Handle of the board
 * @param[in]   b_Port Port to be read. One port includes 4 channels (0..7)
 * @param[out]  b_Value Status of the digital inputs
 * @retval  0   No error
 * @retval -1   The h_DeviceHandle parameter is wrong
 * @retval -2   The b_Port parameter is wrong
 */
_INT_ i_PCIe1032_Read4DigitalInputs(IN  HANDLE h_DeviceHandle,
                                    IN  BYTE b_Port,
                                    OUT BYTE *b_Value);


/* i_PCIe1032_Read8DigitalInputs */
/**
 * Reads 8 digital inputs of the board
 *
 * @param[in]   h_DeviceHandle Handle of the board
 * @param[in]   b_Port Port to be read. One port includes 8 channels (0..3)
 * @param[out]  b_Value Status of the digital inputs
 * @retval  0   No error
 * @retval -1   The h_DeviceHandle parameter is wrong
 * @retval -2   The b_Port parameter is wrong
 */
_INT_ i_PCIe1032_Read8DigitalInputs(IN  HANDLE h_DeviceHandle,
                                    IN  BYTE b_Port,
                                    OUT BYTE *b_Value);


/* i_PCIe1032_Read16DigitalInputs */
/**
 * Reads 16 digital inputs of the board
 *
 * @param[in]   h_DeviceHandle Handle of the board
 * @param[in]   b_Port Port to be read. One port includes 16 channels (0..1)
 * @param[out]  w_Value Status of the digital inputs
 * @retval  0   No error
 * @retval -1   The h_DeviceHandle parameter is wrong
 * @retval -2   The b_Port parameter is wrong
 */
_INT_ i_PCIe1032_Read16DigitalInputs(IN  HANDLE h_DeviceHandle,
                                     IN  BYTE b_Port,
                                     OUT WORD *w_Value);


/* i_PCIe1032_Read32DigitalInputs */
/**
 * Reads the 32 digital inputs of the board
 *
 * @param[in]   h_DeviceHandle Handle of the board
 * @param[out]  dw_Value Status of the digital inputs
 * @retval  0   No error
 * @retval -1   The h_DeviceHandle parameter is wrong
 * @retval -100 An error occurred while calling the IOCTL function
 */
_INT_ i_PCIe1032_Read32DigitalInputs(IN  HANDLE h_DeviceHandle,
                                     OUT DWORD *dw_Value);

/**		@}
 */

/**		\addtogroup EventLogic xPCIe1032 Event logic functions
 * 		\ingroup PCIe1032
 *		@{
 */

/* i_PCIe1032_InitDigitalInputEventLogic */
/**
 * Initializes the input event logic of the board
 *
 * @param[in]   h_DeviceHandle Handle of the board
 * @param[in]   b_InterruptLogic Logic of interrupt (OR -> 0, AND -> 1) (0..1)
 * @param[in]   dw_DigitalInputMode1 Mask of mode 1
 * @param[in]   dw_DigitalInputMode2 Mask of mode 2
 * @retval  0   No error
 * @retval -1   The h_DeviceHandle parameter is wrong
 * @retval -2   The b_InterruptLogic parameter is wrong
 * @retval -100 An error occurred while calling the IOCTL function
 */
_INT_ i_PCIe1032_InitDigitalInputEventLogic(IN  HANDLE h_DeviceHandle,
                                            IN  BYTE b_InterruptLogic,
                                            IN  DWORD dw_DigitalInputMode1,
                                            IN  DWORD dw_DigitalInputMode2);


/* i_PCIe1032_StartDigitalInputEventLogic */
/**
 * Starts the input event logic of the board
 *
 * @param[in]   h_DeviceHandle Handle of the board
 * @retval  0   No error
 * @retval -1   The h_DeviceHandle parameter is wrong
 * @retval -100 An error occurred while calling the IOCTL function
 */
_INT_ i_PCIe1032_StartDigitalInputEventLogic(IN  HANDLE h_DeviceHandle);


/* i_PCIe1032_StopDigitalInputEventLogic */
/**
 * Stops the input event logic of the board
 *
 * @param[in]   h_DeviceHandle Handle of the board
 * @retval  0   No error
 * @retval -1   The h_DeviceHandle parameter is wrong
 * @retval -100 An error occurred while calling the IOCTL function
 */
_INT_ i_PCIe1032_StopDigitalInputEventLogic(IN  HANDLE h_DeviceHandle);


/* i_PCIe1032_ReleaseDigitalInputEventLogic */
/**
 * Releases the input event logic of the board
 *
 * @param[in]   h_DeviceHandle Handle of the board
 * @retval  0   No error
 * @retval -1   The h_DeviceHandle parameter is wrong
 * @retval -100 An error occurred while calling the IOCTL function
 */
_INT_ i_PCIe1032_ReleaseDigitalInputEventLogic(IN  HANDLE h_DeviceHandle);

/**		@}
 */

/**		\addtogroup Timer xPCIe1032 Timer functions
 * 		\ingroup PCIe1032
 *		@{
 */
 
 /* i_PCIe1032_InitAndStartTimer */
/**
 * Initializes and starts timer 0 of the board
 *
 * @param[in]   h_DeviceHandle Handle of the board
 * @param[in]   b_TimeBase Time base of the timer (0 -> microseconds, 1 -> milliseconds, 2 -> seconds) (0..2)
 * @param[in]   w_ReloadValue Timer reload value (1..0xFFF)
 * @param[in]   b_InterruptFlag Interrupt flag (0..1)
 * @retval  0   No error
 * @retval -1   The h_DeviceHandle parameter is wrong
 * @retval -2   The b_TimeBase parameter is wrong
 * @retval -3   The w_ReloadValue parameter is wrong
 * @retval -4   The b_InterruptFlag parameter is wrong
 */
_INT_ i_PCIe1032_InitAndStartTimer(IN  HANDLE h_DeviceHandle,
                                   IN  BYTE b_TimeBase,
                                   IN  WORD w_ReloadValue,
                                   IN  BYTE b_InterruptFlag);


/* i_PCIe1032_ReadTimerValue */
/**
 * Reads the value of the timer 0
 *
 * @param[in]   h_DeviceHandle Handle of the board
 * @param[out]  w_Value Current value of the timer
 * @retval  0   No error
 * @retval -1   The h_DeviceHandle parameter is wrong
 * @retval -100 An error occurred while calling the IOCTL function
 */
_INT_ i_PCIe1032_ReadTimerValue(IN  HANDLE h_DeviceHandle,
                                OUT WORD *w_Value);


/* i_PCIe1032_ReadTimerStatus */
/**
 * Reads the status of timer 0
 *
 * @param[in]   h_DeviceHandle Handle of the board
 * @param[out]  b_SoftwareTriggerStatus Current software trigger status (0 -> No trigger occurred, 1 -> A trigger occurred)
 * @param[out]  w_TimerValue Current value of the timer
 * @retval  0   No error
 * @retval -1   The h_DeviceHandle parameter is wrong
 */
_INT_ i_PCIe1032_ReadTimerStatus(IN  HANDLE h_DeviceHandle,
                                 OUT BYTE *b_TimerStatus,
                                 OUT BYTE *b_SoftwareTriggerStatus,
                                 OUT WORD *w_TimerValue);


/* i_PCIe1032_StopAndReleaseTimer */
/**
 * Stops and releases the timer 0 of the board
 *
 * @param[in]   h_DeviceHandle Handle of the board
 * @retval  0   No error
 * @retval -1   The h_DeviceHandle parameter is wrong
 * @retval -100 An error occurred while calling the IOCTL function
 */
_INT_ i_PCIe1032_StopAndReleaseTimer(IN  HANDLE h_DeviceHandle);


/* i_PCIe1032_TriggerTimer */
/**
 * Triggers the timer 0 of the board
 *
 * @param[in]   h_DeviceHandle Handle of the board
 * @retval  0   No error
 * @retval -1   The h_DeviceHandle parameter is wrong
 * @retval -100 An error occurred while calling the IOCTL function
 */
_INT_ i_PCIe1032_TriggerTimer(IN  HANDLE h_DeviceHandle);

 /**	@}
 */
//##protect##"UserHeaderEnd"
//##protect##"UserHeaderEnd"
/**	@}
 */

#ifdef __cplusplus
}
#endif
