//##protect##"FileHeader"
/** @file PCIe1032.cs
 *
 *  @authors S. Beuque, C. Guinot, J. Krauth, N. Reibel, J. Rose, L. Sartory, S. Weber
 *  @date    Creation: 07.11.2012
 *  @date    $LastChangedDate: 2013-07-25 09:50:50 +0200 (Do, 25. Jul 2013) $
 *  @version $LastChangedRevision: 4417 $
 *
 *  @par DESCRIPTION
 *  PCIe1032 .NET interface functions
 *
 *  @par LICENCE
 *  @verbatim
 *  Copyright (C) 2009 - 2012 ADDI-DATA GmbH
 *
 *  ADDI-DATA GmbH
 *  Airpark Business Center
 *  Airport Boulevard B210
 *  77836 Rheinmnster
 *  Germany
 *  Tel: +49(0)7229/1847-0    |  Fax: +49(0)7229/1847-200
 *  http://www.addi-data-com  |  info@addi-data.com
 *  @endverbatim
 **/
//##protect##"FileHeader"

using System;
using System.Collections.Generic;
using System.Text;
using System.Runtime.InteropServices;

namespace PCIe1032_NET
{
    #region "User declarations"
    //##protect##"UserDeclarations"
    //##protect##"UserDeclarations"
    #endregion

    public class PCIe1032
    {
        private int index;
        private IntPtr handle;

        private DllImport.InterruptFunctionEventHandler interruptFunctionHandler;

        #region "Event Args and Handlers"
        private DigitalInputEventLogicEventArgs digitalInputEventLogicEventArgs;
        public delegate void DigitalInputEventLogicEventHandler(object source, DigitalInputEventLogicEventArgs eventArgs);
        public event DigitalInputEventLogicEventHandler DigitalInputEventLogicEventReceived;

        private TimerEventArgs timerEventArgs;
        public delegate void TimerEventHandler(object source, TimerEventArgs eventArgs);
        public event TimerEventHandler TimerEventReceived;

        //##protect##"EventArgsAndHandlersDeclaration"
        //##protect##"EventArgsAndHandlersDeclaration"
        #endregion
        #region "Constructor/Destructor"
        /// <summary>
        /// Class constructor
        /// </summary>
        private PCIe1032()
        {
            this.index = -1;
            this.handle = IntPtr.Zero;

            // Create the event args (once by board)
            this.digitalInputEventLogicEventArgs = new DigitalInputEventLogicEventArgs();
            this.timerEventArgs = new TimerEventArgs();

            // Create the interrupt handler (once by board)
            this.interruptFunctionHandler = new DllImport.InterruptFunctionEventHandler(InterruptFunction);

            // Tell the garbage collector to keep the pointer
            GC.KeepAlive(this.interruptFunctionHandler);

            //##protect##"Constructor"
            //##protect##"Constructor"
        }

        /// <summary>
        /// Class destructor
        /// </summary>
        ~PCIe1032()
        {
            DllImport.i_PCIe1032_CloseBoard(this.handle);
            this.index = -1;
            this.handle = IntPtr.Zero;

            //##protect##"Destructor"
            //##protect##"Destructor"
        }
        #endregion
        #region "Utils"
        /// <summary>
        /// Creates an instance of a PCIe1032 board
        /// </summary>
        /// <param name="index">The index of the board you want to open</param>
        /// <returns>null on error, instance of PCIe1032 on success</returns>
        public static PCIe1032 OpenBoard(int index)
        {
            PCIe1032 boardInstance;
            int nbBoard;
            IntPtr handle = IntPtr.Zero;

            // Get the number of boards
            nbBoard = GetBoardCount();
            if (nbBoard == -1)
            {
                Console.WriteLine("GetBoardCount failed!");
                return null;
            }

            // Check that the index is in the range
            if (index < 0 || index > nbBoard)
            {
                Console.WriteLine("Invalid index!");
                return null;
            }

            // Try to open the board
            if (DllImport.i_PCIe1032_OpenBoardViaIndex((Byte)index, out handle) != 0)
            {
                Console.WriteLine("i_PCIe1032_OpenBoardViaIndex failed!");
                return null;
            }

            // Create a new instance of the board
            boardInstance = new PCIe1032();

            // Set its parameters
            boardInstance.index = index;
            boardInstance.handle = handle;

            // Return the instance
            return boardInstance;
        }

        /// <summary>
        /// Returns the number of PCIe1032 boards plugged in the computer
        /// </summary>
        /// <returns>The number of boards</returns>
        public static int GetBoardCount()
        {
            int ret;
            byte nbBoard = 0;

            // Call the DLL function
            ret = DllImport.i_PCIe1032_GetNumberOfBoards(out nbBoard);
            if (ret != 0)
                return -1;

            // Success
            return (int)nbBoard;
        }

        /// <summary>
        /// Returns the description of the last error that occurred
        /// </summary>
        /// <returns>A string describing the last error that occurred</returns>
        public string GetLastError()
        {
            // Call the DLL function
            return Marshal.PtrToStringAnsi(DllImport.pc_PCIe1032_GetLastError(this.handle));
        }
        #endregion
        #region "Interrupt handling"
        /// <summary>
        /// Signals to all the listeners that an DigitalInputEventLogic event occurred.
        /// </summary>
        /// <param name="eventArgs">The arguments of the event</param>
        protected virtual void OnDigitalInputEventLogicEventReceived(DigitalInputEventLogicEventArgs eventArgs)
        {
            if (this.DigitalInputEventLogicEventReceived != null)
                this.DigitalInputEventLogicEventReceived(this, eventArgs);
        }

        /// <summary>
        /// Signals to all the listeners that an Timer event occurred.
        /// </summary>
        /// <param name="eventArgs">The arguments of the event</param>
        protected virtual void OnTimerEventReceived(TimerEventArgs eventArgs)
        {
            if (this.TimerEventReceived != null)
                this.TimerEventReceived(this, eventArgs);
        }

        /// <summary>
        /// Interrupt function - Called each time the board has generated an interrupt
        /// </summary>
        private void InterruptFunction(IntPtr handle, Byte overflow, UInt32 source, UInt32 mask, IntPtr args, UInt32 argsSize)
        {
            // DigitalInputEventLogic ?
            if ((source & 0x1) != 0)
            {
                Byte[] data = new Byte[argsSize * sizeof(UInt32)];

                this.digitalInputEventLogicEventArgs.interruptMask = mask;

                // Convert the args
                this.digitalInputEventLogicEventArgs.args = new UInt32[argsSize];
                Marshal.Copy(args, data, 0, (int)argsSize * sizeof(UInt32));
                for (int i = 0; i < argsSize; i++)
                    this.digitalInputEventLogicEventArgs.args[i] = BitConverter.ToUInt32(data, i * sizeof(UInt32));

                // Signal the event
                this.OnDigitalInputEventLogicEventReceived(this.digitalInputEventLogicEventArgs);
            }

            if ((source & 0x4) != 0)
            {
                this.timerEventArgs.interruptMask = mask;

                // Signal the event
                this.OnTimerEventReceived(this.timerEventArgs);
            }


            //##protect##"interruptFunctionBody"
            //##protect##"interruptFunctionBody"
        }

        /// <summary>
        /// Set the board interrupt routine
        /// </summary>
        public int SetBoardIntRoutine()
        {
            return (int)DllImport.i_PCIe1032_SetBoardIntRoutine(this.handle, interruptFunctionHandler);
        }

        /// <summary>
        /// Reset the board interrupt routine
        /// </summary>
        public int ResetBoardIntRoutine()
        {
            return (int)DllImport.i_PCIe1032_ResetBoardIntRoutine(this.handle);
        }

        #endregion

        #region "PCIe1032"

        #endregion

        #region "DigitalInput"

        /// <summary>
        /// Reads 1 digital input of the board
        /// </summary>
        /// <param name="deviceHandle">Handle of the board</param>
        /// <param name="channel">Channel to be read</param>
        /// <param name="value">Status of the digital input</param>
        public int Read1DigitalInput(Byte channel, out Byte value)
        {
            int returnValue;
            //##protect##"functionStart_Read1DigitalInput"
            //##protect##"functionStart_Read1DigitalInput"
            returnValue = DllImport.i_PCIe1032_Read1DigitalInput(this.handle, channel, out value);
            //##protect##"functionEnd_Read1DigitalInput"
            //##protect##"functionEnd_Read1DigitalInput"
            return returnValue;
        }

        /// <summary>
        /// Reads 4 digital inputs of the board
        /// </summary>
        /// <param name="deviceHandle">Handle of the board</param>
        /// <param name="port">Port to be read. One port includes 4 channels</param>
        /// <param name="value">Status of the digital inputs</param>
        public int Read4DigitalInputs(Byte port, out Byte value)
        {
            int returnValue;
            //##protect##"functionStart_Read4DigitalInputs"
            //##protect##"functionStart_Read4DigitalInputs"
            returnValue = DllImport.i_PCIe1032_Read4DigitalInputs(this.handle, port, out value);
            //##protect##"functionEnd_Read4DigitalInputs"
            //##protect##"functionEnd_Read4DigitalInputs"
            return returnValue;
        }

        /// <summary>
        /// Reads 8 digital inputs of the board
        /// </summary>
        /// <param name="deviceHandle">Handle of the board</param>
        /// <param name="port">Port to be read. One port includes 8 channels</param>
        /// <param name="value">Status of the digital inputs</param>
        public int Read8DigitalInputs(Byte port, out Byte value)
        {
            int returnValue;
            //##protect##"functionStart_Read8DigitalInputs"
            //##protect##"functionStart_Read8DigitalInputs"
            returnValue = DllImport.i_PCIe1032_Read8DigitalInputs(this.handle, port, out value);
            //##protect##"functionEnd_Read8DigitalInputs"
            //##protect##"functionEnd_Read8DigitalInputs"
            return returnValue;
        }

        /// <summary>
        /// Reads 16 digital inputs of the board
        /// </summary>
        /// <param name="deviceHandle">Handle of the board</param>
        /// <param name="port">Port to be read. One port includes 16 channels</param>
        /// <param name="value">Status of the digital inputs</param>
        public int Read16DigitalInputs(Byte port, out UInt16 value)
        {
            int returnValue;
            //##protect##"functionStart_Read16DigitalInputs"
            //##protect##"functionStart_Read16DigitalInputs"
            returnValue = DllImport.i_PCIe1032_Read16DigitalInputs(this.handle, port, out value);
            //##protect##"functionEnd_Read16DigitalInputs"
            //##protect##"functionEnd_Read16DigitalInputs"
            return returnValue;
        }

        /// <summary>
        /// Reads the 32 digital inputs of the board
        /// </summary>
        /// <param name="deviceHandle">Handle of the board</param>
        /// <param name="value">Status of the digital inputs</param>
        public int Read32DigitalInputs(out UInt32 value)
        {
            int returnValue;
            //##protect##"functionStart_Read32DigitalInputs"
            //##protect##"functionStart_Read32DigitalInputs"
            returnValue = DllImport.i_PCIe1032_Read32DigitalInputs(this.handle, out value);
            //##protect##"functionEnd_Read32DigitalInputs"
            //##protect##"functionEnd_Read32DigitalInputs"
            return returnValue;
        }

        #endregion
        #region "EventLogic"

        /// <summary>
        /// Initializes the input event logic of the board
        /// </summary>
        /// <param name="deviceHandle">Handle of the board</param>
        /// <param name="interruptLogic">Logic of interrupt (OR -> 0, AND -> 1)</param>
        /// <param name="digitalInputMode1">Mask of mode 1</param>
        /// <param name="digitalInputMode2">Mask of mode 2</param>
        public int InitDigitalInputEventLogic(Byte interruptLogic, UInt32 digitalInputMode1, UInt32 digitalInputMode2)
        {
            int returnValue;
            //##protect##"functionStart_InitDigitalInputEventLogic"
            //##protect##"functionStart_InitDigitalInputEventLogic"
            returnValue = DllImport.i_PCIe1032_InitDigitalInputEventLogic(this.handle, interruptLogic, digitalInputMode1, digitalInputMode2);
            //##protect##"functionEnd_InitDigitalInputEventLogic"
            //##protect##"functionEnd_InitDigitalInputEventLogic"
            return returnValue;
        }

        /// <summary>
        /// Starts the input event logic of the board
        /// </summary>
        /// <param name="deviceHandle">Handle of the board</param>
        public int StartDigitalInputEventLogic()
        {
            int returnValue;
            //##protect##"functionStart_StartDigitalInputEventLogic"
            //##protect##"functionStart_StartDigitalInputEventLogic"
            returnValue = DllImport.i_PCIe1032_StartDigitalInputEventLogic(this.handle);
            //##protect##"functionEnd_StartDigitalInputEventLogic"
            //##protect##"functionEnd_StartDigitalInputEventLogic"
            return returnValue;
        }

        /// <summary>
        /// Stops the input event logic of the board
        /// </summary>
        /// <param name="deviceHandle">Handle of the board</param>
        public int StopDigitalInputEventLogic()
        {
            int returnValue;
            //##protect##"functionStart_StopDigitalInputEventLogic"
            //##protect##"functionStart_StopDigitalInputEventLogic"
            returnValue = DllImport.i_PCIe1032_StopDigitalInputEventLogic(this.handle);
            //##protect##"functionEnd_StopDigitalInputEventLogic"
            //##protect##"functionEnd_StopDigitalInputEventLogic"
            return returnValue;
        }

        /// <summary>
        /// Releases the input event logic of the board
        /// </summary>
        /// <param name="deviceHandle">Handle of the board</param>
        public int ReleaseDigitalInputEventLogic()
        {
            int returnValue;
            //##protect##"functionStart_ReleaseDigitalInputEventLogic"
            //##protect##"functionStart_ReleaseDigitalInputEventLogic"
            returnValue = DllImport.i_PCIe1032_ReleaseDigitalInputEventLogic(this.handle);
            //##protect##"functionEnd_ReleaseDigitalInputEventLogic"
            //##protect##"functionEnd_ReleaseDigitalInputEventLogic"
            return returnValue;
        }

        #endregion
        #region "Timer"

        /// <summary>
        /// Initializes and starts a timer of the board
        /// </summary>
        /// <param name="deviceHandle">Handle of the board</param>
        /// <param name="timeBase">Time base of the timer (0 -> microseconds, 1 -> milliseconds, 2 -> seconds)</param>
        /// <param name="reloadValue">Timer reload value</param>
        /// <param name="interruptFlag">Interrupt flag</param>
        public int InitAndStartTimer(Byte timeBase, UInt16 reloadValue, Byte interruptFlag)
        {
            int returnValue;
            //##protect##"functionStart_InitAndStartTimer"
            //##protect##"functionStart_InitAndStartTimer"
            returnValue = DllImport.i_PCIe1032_InitAndStartTimer(this.handle, timeBase, reloadValue, interruptFlag);
            //##protect##"functionEnd_InitAndStartTimer"
            //##protect##"functionEnd_InitAndStartTimer"
            return returnValue;
        }

        /// <summary>
        /// Reads the value of one of the timer of the board
        /// </summary>
        /// <param name="deviceHandle">Handle of the board</param>
        /// <param name="value">Current value of the timer</param>
        public int ReadTimerValue(out UInt16 value)
        {
            int returnValue;
            //##protect##"functionStart_ReadTimerValue"
            //##protect##"functionStart_ReadTimerValue"
            returnValue = DllImport.i_PCIe1032_ReadTimerValue(this.handle, out value);
            //##protect##"functionEnd_ReadTimerValue"
            //##protect##"functionEnd_ReadTimerValue"
            return returnValue;
        }

        /// <summary>
        /// Reads the status of a timer
        /// </summary>
        /// <param name="deviceHandle">Handle of the board</param>
        /// <param name="timerStatus">Current timer status (0 -> Disabled, 1 -> Enabled)</param>
        /// <param name="softwareTriggerStatus">Current software trigger status (0 -> No trigger occurred, 1 -> A trigger occurred)</param>
        /// <param name="timerValue">Current value of the timer</param>
        public int ReadTimerStatus(out Byte timerStatus, out Byte softwareTriggerStatus, out UInt16 timerValue)
        {
            int returnValue;
            //##protect##"functionStart_ReadTimerStatus"
            //##protect##"functionStart_ReadTimerStatus"
            returnValue = DllImport.i_PCIe1032_ReadTimerStatus(this.handle, out timerStatus, out softwareTriggerStatus, out timerValue);
            //##protect##"functionEnd_ReadTimerStatus"
            //##protect##"functionEnd_ReadTimerStatus"
            return returnValue;
        }

        /// <summary>
        /// Stops and releases a timer of the board
        /// </summary>
        /// <param name="deviceHandle">Handle of the board</param>
        public int StopAndReleaseTimer()
        {
            int returnValue;
            //##protect##"functionStart_StopAndReleaseTimer"
            //##protect##"functionStart_StopAndReleaseTimer"
            returnValue = DllImport.i_PCIe1032_StopAndReleaseTimer(this.handle);
            //##protect##"functionEnd_StopAndReleaseTimer"
            //##protect##"functionEnd_StopAndReleaseTimer"
            return returnValue;
        }

        /// <summary>
        /// Triggers a timer of the board
        /// </summary>
        /// <param name="deviceHandle">Handle of the board</param>
        /// <param name="timerNbr">Number of the timer to trigger</param>
        public int TriggerTimer()
        {
            int returnValue;
            //##protect##"functionStart_TriggerTimer"
            //##protect##"functionStart_TriggerTimer"
            returnValue = DllImport.i_PCIe1032_TriggerTimer(this.handle);
            //##protect##"functionEnd_TriggerTimer"
            //##protect##"functionEnd_TriggerTimer"
            return returnValue;
        }

        #endregion
        #region "User"
        //##protect##"AssemblyUserCode"
        //##protect##"AssemblyUserCode"
        #endregion
    }
}
