using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;

namespace InputEventLogic
{
    public partial class InputEventLogic : Form
    {
        /// <summary>
        /// Board object
        /// </summary>
        private PCIe1032_NET.PCIe1032 board;

        /// <summary>
        /// Counter of interruption
        /// </summary>
        private int interruptCounter;

        /// <summary>
        /// Status of the FIFO overflow
        /// </summary>summary>
        private uint fifoOverflow;
        
        /// <summary>
        /// Mask of input that has generated the interruption
        /// </summary>
        private uint inputMask;

        /// <summary>
        /// Size of the inputStatus
        /// </summary>
        private uint size;

        /// <summary>
        /// Status of the input
        /// </summary>
        private UInt32 inputStatus;

        /// <summary>
        /// Constructor
        /// </summary>
        public InputEventLogic()
        {
            InitializeComponent();

            // Reset the parameters
            this.interruptCounter = 0;
            this.fifoOverflow = 0;
            this.inputMask = 0;
            this.inputStatus = 0;
        }

        /// <summary>
        /// Occurs when the form loads for the first time
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void InputEventLogic_Load(object sender, EventArgs e)
        {
            // Open the board
            this.board = PCIe1032_NET.PCIe1032.OpenBoard(0);

            // Check it
            if (this.board == null)
            {
                MessageBox.Show("Error while opening board 0");
                this.Close();
                return;
            }

            // Set the interrupt routine callback
            if (this.board.SetBoardIntRoutine() != 0)
            {
                MessageBox.Show("Error while setting interrupt routine");
                this.Close();
                return;
            }

            // Set an event listener
            this.board.DigitalInputEventLogicEventReceived += new PCIe1032_NET.PCIe1032.DigitalInputEventLogicEventHandler(board_DigitalInputEventLogicEventReceived);
        }

        /// <summary>
        /// Listener to the digital input event
        /// </summary>
        /// <param name="source"></param>
        /// <param name="args"></param>
        private void board_DigitalInputEventLogicEventReceived(object source, PCIe1032_NET.DigitalInputEventLogicEventArgs args)
        {
            this.interruptCounter++;
            if (args.fifoOverflow == false)
                this.fifoOverflow = 0;
            if (args.fifoOverflow == true)
                this.fifoOverflow = 1;
            this.inputMask = args.interruptMask;
            this.size = args.size;
            this.inputStatus = args.args[0];
        }

        /// <summary>
        /// Click on the start button
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void start_Button_Click(object sender, EventArgs e)
        {
            UInt16 interruptMask1 = 0;
            UInt16 interruptMask2 = 0;

            // Get the mode 1 and 2 values
            interruptMask1 = Convert.ToUInt16(this.interruptMask1TextBox.Text);
            interruptMask2 = Convert.ToUInt16(this.interruptMask2TextBox.Text);
            if (interruptMask1 > 0xFFFF)
            {
                MessageBox.Show("Invalid value for mode 1, using 1 instead. ");
                interruptMask1 = 1;
                interruptMask1TextBox.Text = "1";
            }
            if (interruptMask2 > 0xFFFF)
            {
                MessageBox.Show("Invalid value for mode 2, using 1 instead. ");
                interruptMask2 = 1;
                interruptMask2TextBox.Text = "1";
            }

            // Init event logic
            if (this.board.InitDigitalInputEventLogic((byte)interruptLogicComboBox.SelectedIndex, interruptMask1, interruptMask2) != 0)
            {
                MessageBox.Show("Error PCIe1032_InitDigitalInputEventLogic");
                return;
            }

            // Start event logic
            if (this.board.StartDigitalInputEventLogic() != 0)
            {
                MessageBox.Show("Error PCIe1032_StartDigitalInputEventLogic");
                return;
            }

            // Reset the counter value
            this.interruptCounter = 0;

            // Reset other parameters
            this.fifoOverflow = 0;
            this.inputMask = 0;
            this.inputStatus = 0;

            // Enables/disables buttons
            this.startButton.Enabled = false;
            this.stopButton.Enabled = true;
            interruptLogicComboBox.Enabled = false;
            interruptMask1TextBox.Enabled = false;
            interruptMask2TextBox.Enabled = false;

            // Start the display timer
            displayTimer.Start();
        }

        /// <summary>
        /// Display timer tick
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void displayTimer_Tick(object sender, EventArgs e)
        {
            this.interruptCounterLabel.Text = this.interruptCounter.ToString();
            this.fifoOverflowLabel.Text = this.fifoOverflow.ToString();
            this.inputMaskLabel.Text = this.inputMask.ToString();
            this.inputStatusLabel.Text = this.inputStatus.ToString();
        }

        /// <summary>
        /// Click on stop
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void stopButton_Click(object sender, EventArgs e)
        {
            // Stop the display timer
            displayTimer.Stop();

            // Stop input event logic
            if (this.board.StopDigitalInputEventLogic() != 0)
            {
                MessageBox.Show("StopDigitalInputEventLogic error");
            }

            // Release it
            if (this.board.ReleaseDigitalInputEventLogic() != 0)
            {
                MessageBox.Show("ReleaseDigitalInputEventLogic error");
            }

            // Enables/disables buttons
            this.stopButton.Enabled = false;
            this.startButton.Enabled = true;
            interruptLogicComboBox.Enabled = true;
            interruptMask1TextBox.Enabled = true;
            interruptMask2TextBox.Enabled = true;
        }

        /// <summary>
        /// Occurs just before the form is closed
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void InputEventLogic_FormClosing(object sender, FormClosingEventArgs e)
        {
           // Reset the board interrupt routine
            board.ResetBoardIntRoutine();
        }
    }
}
