using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;

namespace TimerInterrupt
{
    public partial class TimerInterrupt : Form
    {
        /// <summary>
        /// Handle to the first board
        /// </summary>
        PCIe1032_NET.PCIe1032 board;

        /// <summary>
        /// Some internal value. Store the number of interrupt per component
        /// </summary>
        private int timer1InterruptNb = 0;

        /// <summary>
        /// Constructor of the form
        /// </summary>
        public TimerInterrupt()
        {
            InitializeComponent();
        }

        /// <summary>
        /// Called at the first load
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void TimerInterrupt_Load(object sender, EventArgs e)
        {
            int ret = 0;

            reloadValueTextBox.Text = "1000";

            // Load the saved settings
            timeBaseComboBox.SelectedIndex = Properties.Settings.Default.timeBase;
            reloadValueTextBox.Text = Properties.Settings.Default.reloadValue.ToString();
            interruptComboBox.SelectedIndex = Properties.Settings.Default.interrupt;


            // Open the first board of the system
            this.board = PCIe1032_NET.PCIe1032.OpenBoard(0);
            // Error handling
            if (this.board == null)
            {
                MessageBox.Show("Error OpenBoard returned null");
                this.Close();
                return;
            }

            // Enable the interruption
            if ((ret = this.board.SetBoardIntRoutine()) != 0)
            {
                MessageBox.Show("Error " + ret.ToString() + " in SetBoardIntRoutine");
                this.Close();
                return;
            }

            // Stop the components. No check since if nothing is started the function will failed
            this.board.StopAndReleaseTimer();

            // Listen to the counter timer watchdog interruption
            this.board.TimerEventReceived += new PCIe1032_NET.PCIe1032.TimerEventHandler(board_TimerEventReceived);
        }

        void board_TimerEventReceived(object source, PCIe1032_NET.TimerEventArgs args)
        {
            // Increment the interrupt counter
            if ((args.interruptMask) != 0)
            {
                this.timer1InterruptNb++;
            }
        }

        /// <summary>
        /// Display timer for timer 1 ticks
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void timer1DisplayTimer_Tick(object sender, EventArgs e)
        {
            int ret = 0;
            byte status = 0;
            byte softwareTriggerStatus = 0;
            ushort value = 0;

            // Read the value
            if ((ret = this.board.ReadTimerStatus(out status, out softwareTriggerStatus, out value)) != 0)
            {
                MessageBox.Show("Error " + ret.ToString() + " in ReadTimerStatus");
                return;
            }
            timer1Value.Text = value.ToString();
            timer1Interrupt.Text = this.timer1InterruptNb.ToString();
        }

        /// <summary>
        /// Occurs while the form is closing
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void TimerInterrupt_FormClosing(object sender, FormClosingEventArgs e)
        {
            int ret = 0;
            UInt16 reloadValue;

            // Save the settings
            Properties.Settings.Default.timeBase = timeBaseComboBox.SelectedIndex;
            Properties.Settings.Default.interrupt = interruptComboBox.SelectedIndex;
            // Get the reload value
            reloadValue = Convert.ToUInt16(reloadValueTextBox.Text);
            if (reloadValue > 4095 || reloadValue < 1)
            {
                MessageBox.Show("Reload value invalid, using 1000 instead. ");
                reloadValue = 1000;
                reloadValueTextBox.Text = "1000";
            }
            Properties.Settings.Default.reloadValue = reloadValue;
            Properties.Settings.Default.Save();

            // Stop the timers
            this.board.StopAndReleaseTimer();

            // Disable the interruption
            if ((ret = this.board.ResetBoardIntRoutine()) != 0)
            {
                MessageBox.Show("Error " + ret.ToString() + " in ResetBoardIntRoutine");
                return;
            }
        }

        /// <summary>
        /// Click on the start button for the timer 1
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void timer1Start_Click(object sender, EventArgs e)
        {
            int ret = 0;
            UInt16 reloadValue;

            // Initialize the value of the interrupt count
            this.timer1InterruptNb = 0;

            // Get the reload value
            reloadValue = Convert.ToUInt16(reloadValueTextBox.Text);
            if (reloadValue > 4095)
            {
                MessageBox.Show("Reload value invalid, using 1000 instead. ");
                reloadValue = 1000;
                reloadValueTextBox.Text = "1000";
            }

            // Initialize and start the timer
            if ((ret = this.board.InitAndStartTimer((byte)timeBaseComboBox.SelectedIndex, reloadValue, (byte)interruptComboBox.SelectedIndex)) != 0)
            {
                MessageBox.Show("Error " + ret.ToString() + " in InitAndStartTimer");
                return;
            }
            // Disable/enable button
            timeBaseComboBox.Enabled = false;
            reloadValueTextBox.Enabled = false;
            interruptComboBox.Enabled = false;
            timer1Start.Enabled = false;
            timer1Stop.Enabled = true;
            timer1Trigger.Enabled = true;

            // Start the display timer
            timer1DisplayTimer.Start();
        }

        /// <summary>
        /// Click on the stop button for the timer 1
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void timer1Stop_Click(object sender, EventArgs e)
        {
            int ret = 0;
            // Stop the timer
            if ((ret = this.board.StopAndReleaseTimer()) != 0)
            {
                MessageBox.Show("Error " + ret.ToString() + " in StopTimer");
                return;
            }
            // Disable/enable button
            timeBaseComboBox.Enabled = true;
            reloadValueTextBox.Enabled = true;
            interruptComboBox.Enabled = true;
            timer1Start.Enabled = true;
            timer1Stop.Enabled = false;
            timer1Trigger.Enabled = false;
            // Stop the display timer
            timer1DisplayTimer.Stop();
        }

        /// <summary>
        /// Click on the trigger button for the timer 1
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void timer1Trigger_Click(object sender, EventArgs e)
        {
            int ret = 0;

            // Trigger the timer
            if ((ret = this.board.TriggerTimer()) != 0)
            {
                MessageBox.Show("Error " + ret.ToString() + " in TriggerTimer");
                return;
            }
        }
    }
}